import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class Hessian {

    public static double denominator(double x1,double y1,double x2,double y2) {
	double d =(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2);
	return(d);
    }

    /**Inverse square function**/

    public static double F0(double x1,double y1,double x2,double y2) {
	double n=(1+x1*x1+y1*y1)*(1+x2*x2+1*y2*y2);
	double d = 4*denominator(x1,y1,x2,y2);
	return(n/d);
    }

    public static double G0(double x,double y) {
	double t=1+x*x+y*y;
	return(t/4.0);
    }


    /**First partial derivatives**/

    public static double F1(int q,double x1,double y1,double x2,double y2) {
	if(q==1) return(F1(x1,y1,x2,y2));
       	if(q==2) return(F2(x1,y1,x2,y2));		
                  if(q==3) return(F3(x1,y1,x2,y2));
	if(q==4) return(F4(x1,y1,x2,y2));
	return(0);
    }

    public static double G1(int q,double x,double y) {
	if(q==1) return(x/2);
	if(q==2) return(y/2);
	return(0);
    }

    public static double F1(double x1,double y1,double x2,double y2) {
	double a1=1+x2*x2+y2*y2;
	double a2=-x1+x2-x1*x1*x2+x1*x2*x2+x2*y1*y1-2*x1*y1*y2+x1*y2*y2;
	double d=denominator(x1,y1,x2,y2);
	double t=a1*a2/(2*d*d);
	return(t);
    }

    public static double F2(double x1,double y1,double x2,double y2) {
	return(F1(y1,x1,y2,x2));
    }

    public static double F3(double x1,double y1,double x2,double y2) {
	return(F1(x2,y2,x1,y1));
    }

    public static double F4(double x1,double y1,double x2,double y2) {
	return(F1(y2,x2,y1,x1));
    }


    /**Second partial derivatives**/

    public static double G2(int i,int j,double x,double y) {
	if(i==j) return(0.5);
	return(0);
    }

    public static double F2(int i,int j,double x1,double y1,double x2,double y2) {
	if((i==1)&&(j==1)) return(F11(x1,y1,x2,y2));
	if((i==1)&&(j==2)) return(F12(x1,y1,x2,y2));
	if((i==1)&&(j==3)) return(F13(x1,y1,x2,y2));
	if((i==1)&&(j==4)) return(F14(x1,y1,x2,y2));

	if((i==2)&&(j==1)) return(F21(x1,y1,x2,y2));
	if((i==2)&&(j==2)) return(F22(x1,y1,x2,y2));
	if((i==2)&&(j==3)) return(F23(x1,y1,x2,y2));
	if((i==2)&&(j==4)) return(F24(x1,y1,x2,y2));

	if((i==3)&&(j==1)) return(F31(x1,y1,x2,y2));
	if((i==3)&&(j==2)) return(F32(x1,y1,x2,y2));
	if((i==3)&&(j==3)) return(F33(x1,y1,x2,y2));
	if((i==3)&&(j==4)) return(F34(x1,y1,x2,y2));

	if((i==4)&&(j==1)) return(F41(x1,y1,x2,y2));
	if((i==4)&&(j==2)) return(F42(x1,y1,x2,y2));
	if((i==4)&&(j==3)) return(F43(x1,y1,x2,y2));
	if((i==4)&&(j==4)) return(F44(x1,y1,x2,y2));

	return(0);
    }

    public static double F11(double x1,double y1,double x2,double y2) {
	double a1=1+x2*x2+y2*y2;
	double[] a2=new double[20];

	a2[0]= + 3*x1*x1;
	a2[1]= - 6*x1*x2;
	a2[2]= + 2*x1*x1*x1*x2;
	a2[3]= + 3*x2*x2;
	a2[4]= - 3*x1*x1*x2*x2;
	a2[5]=+ x2*x2*x2*x2;
	a2[6]= - y1*y1;
	a2[7]= - 6*x1*x2*y1*y1;
	a2[8]= + 5*x2*x2*y1*y1;
	a2[9]= + 2*y1*y2;
	a2[10]= +  6*x1*x1*y1*y2;
	a2[11]= - 4*x2*x2*y1*y2;
	a2[12]= - 2*y1*y1*y1*y2;
	a2[13]= -  y2*y2;
	a2[14]= - 3*x1*x1*y2*y2;
	a2[15]=+ 2*x2*x2*y2*y2;
	a2[16]=+  5*y1*y1*y2*y2;
	a2[17]= - 4*y1*y2*y2*y2;
                  a2[18]= + y2*y2*y2*y2;

	double a3=0;
	for(int i=0;i<19;++i) a3=a3+a2[i];

	double d=denominator(x1,y1,x2,y2);
	double t=a1*a3/(2*d*d*d);
	return(t);
    }


    public static double F22(double x1,double y1,double x2,double y2) {
	return(F11(y1,x1,y2,x2));
    }

    public static double F33(double x1,double y1,double x2,double y2) {
	return(F11(x2,y2,x1,y1));
    }

    public static double F44(double x1,double y1,double x2,double y2) {
	return(F11(y2,x2,y1,x1));
    }


    public static double F13(double x1,double y1,double x2,double y2) {
  	double[] a1=new double[42];
	a1[0]=-3*x1*x1;
	a1[1]= - x1*x1*x1*x1;
	a1[2]= + 6*x1*x2;
	a1[3]= - 2*x1*x1*x1*x2;
	a1[4]= - 3*x2*x2;
	a1[5]=+ 6*x1*x1*x2*x2;
	a1[6]= - 3*x1*x1*x1*x1*x2*x2;
	a1[7]= - 2*x1*x2*x2*x2;
	a1[8]=+ 6*x1*x1*x1*x2*x2*x2; 
	a1[9]= - x2*x2*x2*x2;
	a1[10]=- 3*x1*x1*x2*x2*x2*x2;
	a1[11]= + y1*y1;
                  a1[12]=+ 2*x1*x2*y1*y1;
	a1[13]= +  2*x1*x2*x2*x2*y1*y1; 
                  a1[14]= - x2*x2*x2*x2*y1*y1;
                  a1[15]= + y1*y1*y1*y1;
	a1[16]= +     3*x2*x2*y1*y1*y1*y1;
	a1[17]= - 2*y1*y2;
                  a1[18]= - 6*x1*x1*y1*y2;
	a1[19]=+8*x1*x2*y1*y2;
	a1[20]= - 4*x1*x1*x1*x2*y1*y2;
	a1[21]= - 6*x2*x2*y1*y2;
	a1[22]= + 6*x1*x1*x2*x2*y1*y2;
	a1[23]= - 4*x1*x2*x2*x2*y1*y2;
	a1[24]= - 2*y1*y1*y1*y2;
	a1[25]= - 4*x1*x2*y1*y1*y1*y2;
	a1[26]= - 6*x2*x2*y1*y1*y1*y2;
	a1[27]= + y2*y2;
                  a1[28]= - x1*x1*x1*x1*y2*y2;
	a1[29]= +2*x1*x2*y2*y2;
	a1[30]= + 2*x1*x1*x1*x2*y2*y2;
	a1[31]= +     2*y1*y1*y2*y2;
	a1[32]=+ 14*x1*x2*y1*y1*y2*y2;
	a1[33]= +     y1*y1*y1*y1*y2*y2;
	a1[34]= - 2*y1*y2*y2*y2;
	a1[35]=-  6*x1*x1*y1*y2*y2*y2;
	a1[36]= - 4*x1*x2*y1*y2*y2*y2;
	a1[37]= -     2*y1*y1*y1*y2*y2*y2;
	a1[38]= + y2*y2*y2*y2;
	a1[39]= + 3*x1*x1*y2*y2*y2*y2;
	a1[40]= + y1*y1*y2*y2*y2*y2;
	double a2=0;
	for(int i=0;i<41;++i) a2=a2+a1[i];
	double d=denominator(x1,y1,x2,y2);
	double t=a2/(2*d*d*d);
	return(t);
    }


    public static double F31(double x1,double y1,double x2,double y2) {
	return(F13(x1,y1,x2,y2));
    }

    public static double F24(double x1,double y1,double x2,double y2) {
	return(F13(y1,x1,y2,x2));
    }

    public static double F42(double x1,double y1,double x2,double y2) {
	return(F13(y1,x1,y2,x2));
    }


    public static double F12(double x1,double y1,double x2,double y2) {
	double a1=1+x2*x2+y2*y2;
	double[] a2=new double[20];

	a2[0]=  2*x1*y1;
	a2[1]= - 2*x2*y1;
	a2[2]= + 3*x1*x1*x2*y1;
	a2[3]= - 4*x1*x2*x2*y1;
	a2[4]= +x2*x2*x2*y1;
	a2[5]= - x2*y1*y1*y1;
	a2[6]= - 2*x1*y2;
	a2[7]= - x1*x1*x1*y2;
	a2[8]= + 2*x2*y2 ;
	a2[9]=  +  x1*x2*x2*y2;
	a2[10]= 3*x1*y1*y1*y2;
	a2[11]= - 4*x1*y1*y2*y2;
	a2[12]= + x2*y1*y2*y2;
	a2[13]= + x1*y2*y2*y2;

	double a3=0;
	for(int i=0;i<14;++i) a3=a3+a2[i];

	double d=denominator(x1,y1,x2,y2);
	double t=a1*a3/(d*d*d);
	return(t);
    }

    public static double F21(double x1,double y1,double x2,double y2) {
	return(F12(x1,y1,x2,y2));
    }

    public static double F34(double x1,double y1,double x2,double y2) {
	return(F12(x2,y2,x1,y1));
    }

    public static double F43(double x1,double y1,double x2,double y2) {
	return(F12(x2,y2,x1,y1));
    }


    public static double F14(double x1,double y1,double x2,double y2) {
  	double[] a1=new double[42];

	a1[0]= -2*x1*y1;
	a1[1]= - x1*x1*x1*y1 ;
	a1[2]=+ 2*x2*y1 ;
	a1[3]=- x1*x2*x2*y1 ;
	a1[4]=- x1*x1*x1*x2*x2*y1 ;
	a1[5]=+ 2*x2*x2*x2*y1 ;
	a1[6]=+ x1*x2*x2*x2*x2*y1 ;
	a1[7]=- x1*y1*y1*y1 ;
	a1[8]=+ 2*x2*y1*y1*y1 ;
	a1[9]=- x1*x2*x2*y1*y1*y1 ;
	a1[10]=+ 2*x2*x2*x2*y1*y1*y1 ;
	a1[11]=+ 2*x1*y2 - 2*x2*y2 ;
	a1[12]=+ 3*x1*x1*x2*y2 ;
	a1[13]=- x1*x1*x1*x1*x2*y2 ;
	a1[14]=- 2*x1*x2*x2*y2 ;
	a1[15]=+ 4*x1*x1*x1*x2*x2*y2 ;
	a1[16]=- x2*x2*x2*y2 ;
	a1[17]=- 3*x1*x1*x2*x2*x2*y2 ;
	a1[18]=- 2*x1*y1*y1*y2 ;
	a1[19]=- x2*y1*y1*y2 ;
	a1[20]=- 2*x1*x2*x2*y1*y1*y2 ;
	a1[21]=- x2*x2*x2*y1*y1*y2 ;
	a1[22]=+ x2*y1*y1*y1*y1*y2 ;
	a1[23]=+ 3*x1*y1*y2*y2 ;
	a1[24]=- 3*x1*x1*x1*y1*y2*y2 ;
	a1[25]=+ 6*x1*x1*x2*y1*y2*y2 ;
	a1[26]=- 3*x1*y1*y1*y1*y2*y2 ;
	a1[27]=+ 2*x1*x1*x1*y2*y2*y2 ;
	a1[28]=- x2*y2*y2*y2 ;
	a1[29]=- 3*x1*x1*x2*y2*y2*y2 ;
	a1[30]=+ 4*x1*y1*y1*y2*y2*y2 ;
	a1[31]=- x2*y1*y1*y2*y2*y2 ;
	a1[32]=- x1*y1*y2*y2*y2*y2;

	double a2=0;
	for(int i=0;i<33;++i) a2=a2+a1[i];
	double d=denominator(x1,y1,x2,y2);
	double t=a2/(d*d*d);
	return(t);
    }

    public static double F41(double x1,double y1,double x2,double y2) {
	return(F14(x1,y1,x2,y2));
    }

    public static double F23(double x1,double y1,double x2,double y2) {
	return(F14(y1,x1,y2,x2));
    }

    public static double F32(double x1,double y1,double x2,double y2) {
	return(F14(y1,x1,y2,x2));
    }



    /**General Exponent First Partials**/

    public static double F1(int q,double ee,double x1,double y1,double x2,double y2) {
	double e=ee/2.0;
	double x=F0(x1,y1,x2,y2);
	double a1=e*Math.pow(x,e-1);
	double a2=F1(q,x1,y1,x2,y2);
	return(a1*a2);
    }

    public static double F2(int a,int b,double ee,double x1,double y1,double x2,double y2) {
	double e=ee/2.0;
	double x=F0(x1,y1,x2,y2);
	double a1=e*Math.pow(x,e-1);
	double a2=e*(e-1)*Math.pow(x,e-2);
	double b1=F1(a,x1,y1,x2,y2);
	double b2=F1(b,x1,y1,x2,y2);
	double c=F2(a,b,x1,y1,x2,y2);
	double t=a2*b1*b2+a1*c;

	return(t);
    }


    public static double G1(int q,double ee,double x1,double y1) {
	double e=ee/2.0;
	double x=G0(x1,y1);
	double a1=e*Math.pow(x,e-1);
	double a2=G1(q,x1,y1);
	return(a1*a2);
    }

    public static double G2(int a,int b,double ee,double x1,double y1) {
	double e=ee/2.0;
	double x=G0(x1,y1);
	double a1=e*Math.pow(x,e-1);
	double a2=e*(e-1)*Math.pow(x,e-2);
	double b1=G1(a,x1,y1);
	double b2=G1(b,x1,y1);
	double c=G2(a,b,x1,y1);
	double t=a2*b1*b2+a1*c;	
	return(t);
    }


    public static int[][] pos() {
	int[][] p={{1,0,0,0,1,1,1,0,0,0},{2,0,0,0,2,2,2,0,0,0},{0,1,0,0,3,0,0,1,1,0},{0,2,0,0,4,0,0,2,2,0},{0,0,1,0,0,3,0,3,0,1},{0,0,2,0,0,4,0,4,0,2},{0,0,0,1,0,0,3,0,3,3},{0,0,0,2,0,0,4,0,4,4}};
	return(p);
    }

    public static int[][] var() {
	int[][] v={{0,1,-1,-1},{2,3,-1,-1},{4,5,-1,-1},{6,7,-1,-1},{0,1,2,3},{0,1,4,5},{0,1,6,7},{2,3,4,5},{2,3,6,7},{4,5,6,7}};
	return(v);
    }

    public static double hessian(int aa,int bb,double e,double[] y) {

	double[] x=new double[8];
	x[0]=y[0];
	x[1]=0;
	for(int i=2;i<8;++i) x[i]=y[i-1];

	int a=aa;
	int b=bb;
	if(a>0) ++a;
	if(b>0) ++b;

	double total=0;
	int[][] p=pos();
	int[][] v=var();
	for(int i=0;i<10;++i) {
	    int A=p[a][i];
	    int B=p[b][i];
	    if(i<4) {
		if((A>=1)&&(A<=2)) {
		    if((B>=1)&&(B<=2)) {
			double g=G2(A,B,e,x[v[i][0]],x[v[i][1]]);
			total=total+g;
		    }
		}
	    }
	    if(i>=4) {
		if((A>=1)&&(A<=4)) {
		    if((B>=1)&&(B<=4)) {	
	                                    double f=F2(A,B,e,x[v[i][0]],x[v[i][1]],x[v[i][2]],x[v[i][3]]);
	                                    total=total+f;
		    }
		    }
	    }
	}
	    return(total);
    }


    public static double[][] hessian(double e,double[] y) {
	double[][] h=new double[7][7];
	for(int i=0;i<7;++i) {
	    for(int j=0;j<7;++j) {
		h[i][j]=hessian(i,j,e,y);
	    }
	}
	return(h);
    }


}

